@@ -24,8 +24,11 @@ module Control.Monad.Trans.Identity (
 
 import Control.Applicative
 import Control.Monad (MonadPlus(mzero, mplus))
+import Control.Monad.Fix (MonadFix(mfix))
 import Control.Monad.IO.Class (MonadIO(liftIO))
 import Control.Monad.Trans.Class (MonadTrans(lift))
+import Data.Foldable (Foldable(foldMap))
+import Data.Traversable (Traversable(traverse))
 
 -- | The trivial monad transformer, which maps a monad to an equivalent monad.
 newtype IdentityT m a = IdentityT { runIdentityT :: m a }
@@ -33,10 +36,16 @@ newtype IdentityT m a = IdentityT { runIdentityT :: m a }
 instance (Functor m) => Functor (IdentityT m) where
     fmap f = mapIdentityT (fmap f)
 
+instance (Foldable f) => Foldable (IdentityT f) where
+    foldMap f (IdentityT a) = foldMap f a
+
+instance (Traversable f) => Traversable (IdentityT f) where
+    traverse f (IdentityT a) = IdentityT <$> traverse f a
+
 instance (Applicative m) => Applicative (IdentityT m) where
     pure x = IdentityT (pure x)
     (<*>) = lift2IdentityT (<*>)
- 
+
 instance (Alternative m) => Alternative (IdentityT m) where
     empty = IdentityT empty
     (<|>) = lift2IdentityT (<|>)
@@ -45,11 +54,14 @@ instance (Monad m) => Monad (IdentityT m) where
     return = IdentityT . return
     m >>= k = IdentityT $ runIdentityT . k =<< runIdentityT m
     fail msg = IdentityT $ fail msg
- 
+
 instance (MonadPlus m) => MonadPlus (IdentityT m) where
     mzero = IdentityT mzero
     mplus = lift2IdentityT mplus
 
+instance (MonadFix m) => MonadFix (IdentityT m) where
+    mfix f = IdentityT (mfix (runIdentityT . f))
+
 instance (MonadIO m) => MonadIO (IdentityT m) where
     liftIO = IdentityT . liftIO
 
