@@ -9,11 +9,11 @@
 -- Stability   :  experimental
 -- Portability :  portable
 --
--- Lazy state monads, passing an updateable state through a computation.
+-- Lazy state monads, passing an updatable state through a computation.
 -- See below for examples.
 --
--- In this version, sequencing of computations is lazy in the state.
--- For a strict version, see "Control.Monad.Trans.Writer.Strict", which
+-- In this version, sequencing of computations is lazy.
+-- For a strict version, see "Control.Monad.Trans.State.Strict", which
 -- has the same interface.
 --
 -- Some computations may not require the full power of state transformers:
@@ -79,9 +79,10 @@ type State s = StateT s Identity
 
 -- | Construct a state monad computation from a function.
 -- (The inverse of 'runState'.)
-state :: (s -> (a, s))  -- ^pure state transformer
-      -> State s a      -- ^equivalent state-passing computation
-state f = StateT (Identity . f)
+state :: Monad m
+      => (s -> (a, s))  -- ^pure state transformer
+      -> StateT s m a   -- ^equivalent state-passing computation
+state f = StateT (return . f)
 
 -- | Unwrap a state monad computation as a function.
 -- (The inverse of 'state'.)
@@ -179,7 +180,7 @@ instance (Functor m, MonadPlus m) => Alternative (StateT s m) where
     (<|>) = mplus
 
 instance (Monad m) => Monad (StateT s m) where
-    return a = StateT $ \s -> return (a, s)
+    return a = state $ \s -> (a, s)
     m >>= k  = StateT $ \s -> do
         ~(a, s') <- runStateT m s
         runStateT (k a) s'
@@ -202,27 +203,25 @@ instance (MonadIO m) => MonadIO (StateT s m) where
 
 -- | Fetch the current value of the state within the monad.
 get :: (Monad m) => StateT s m s
-get = StateT $ \s -> return (s, s)
+get = state $ \s -> (s, s)
 
 -- | @'put' s@ sets the state within the monad to @s@.
 put :: (Monad m) => s -> StateT s m ()
-put s = StateT $ \_ -> return ((), s)
+put s = state $ \_ -> ((), s)
 
 -- | @'modify' f@ is an action that updates the state to the result of
 -- applying @f@ to the current state.
+--
+-- * @'modify' f = 'get' >>= ('put' . f)@
 modify :: (Monad m) => (s -> s) -> StateT s m ()
-modify f = do
-    s <- get
-    put (f s)
+modify f = state $ \s -> ((), f s)
 
 -- | Get a specific component of the state, using a projection function
 -- supplied.
 --
 -- * @'gets' f = 'liftM' f 'get'@
 gets :: (Monad m) => (s -> a) -> StateT s m a
-gets f = do
-    s <- get
-    return (f s)
+gets f = state $ \s -> (f s, s)
 
 -- | Uniform lifting of a @callCC@ operation to the new monad.
 -- This version rolls back to the original state on entering the
@@ -288,9 +287,9 @@ For example, item can be written as:
 
 {- $counting
 
-A function to increment a counter.  Taken from the paper
-/Generalising Monads to Arrows/, John
-Hughes (<http://www.math.chalmers.se/~rjmh/>), November 1998:
+A function to increment a counter.
+Taken from the paper \"Generalising Monads to Arrows\",
+John Hughes (<http://www.cse.chalmers.se/~rjmh/>), November 1998:
 
 > tick :: State Int Int
 > tick = do n <- get
